@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# $Id: clamav-update.pl,v 1.10 2007/02/11 14:42:18 okamura Exp $
+# $Id: clamav-update.pl,v 1.22 2007/03/17 17:15:35 okamura Exp $
 
 =head1 NAME
 
@@ -59,17 +59,32 @@ use vars qw(
 # サブルーチン
 #-------------------------------------------------------------------------------
 sub GetOptionValue {
-	my	($name) = @_;
+	my	@name = @_;
+	my	$ref;
 
-	return $Option{$name}	if (
-		exists $Option{$name} and defined($Option{$name})
-	);
-	return $Setting{option}->{$name}	if (
-		exists $Setting{option}->{$name}
-			and
-		defined($Setting{option}->{$name})
-	);
-	return undef;
+	$ref = \%Option;
+	foreach (@name) {
+		unless (exists($ref->{$_}) and defined($ref->{$_})) {
+			$ref = undef;
+		}
+	}
+	continue {
+		$ref = $ref->{$_};
+	}
+
+	return $ref	if (defined($ref));
+
+	$ref = $Setting{option};
+	foreach (@name) {
+		unless (exists($ref->{$_}) and defined($ref->{$_})) {
+			$ref = undef;
+		}
+	}
+	continue {
+		$ref = $ref->{$_};
+	}
+
+	return $ref;
 }
 
 sub Log;
@@ -479,6 +494,16 @@ sub CheckSecurity {
 #	3. do ./configure
 #	4. do make
 #	5. do make install
+#	REQUIRED:
+#		download file name format is xxxx-1.2.3.tar.gz
+#		extracted directory name format is xxxx-1.2.3
+#	SAMPLE CONFIGUREATION:
+#		$Setting{version}->{latest}->{command} = <mthod of your software>;
+#		$Setting{version}->{latest}->{regexp} = qr/regexp for above/;
+#		$Setting{version}->{current}->{command} = <mthod of your software>;
+#		$Setting{version}->{current}->{regexp} = qr/regexp for above/;
+#		$Setting{option}->{src} = '<Base URL for your software>';
+#		$Setting{option}->{name} = '<Name of your software>';
 sub PhaseSpecifier4General {
 	#### Initialization ####
 	# Delete any from phase definitions.
@@ -585,15 +610,19 @@ sub PhaseSpecifier4ClamAV4ClamXav {
 	#### Install Phase Customization ####
 	# add to command sequence
 	push @{$Phase{install}->{method}}, (
-		[qw(install -d -o root -g admin -m 0755 /usr/local/clamXav)],
-		[qw(chown -R root:admin /usr/local/clamXav)],
-		[qw(chmod 664 /usr/local/clamXav/etc/freshclam.conf)],
+		[qw(install -d -o root -g admin -m 0775 /usr/local/clamXav)],
+		[qw(chown -R root:admin /usr/local/clamXav/etc)],
+		[qw(find /usr/local/clamXav/etc -type d -exec chmod 0775 {} \;)],
+		[qw(find /usr/local/clamXav/etc -type f -exec chmod 0664 {} \;)],
+		[qw(chown -R root:admin /usr/local/clamXav/bin)],
+		[qw(chmod -R 0755 /usr/local/clamXav/bin)],
 		[qw(chown clamav /usr/local/clamXav/bin/freshclam)],
 		[qw(chmod u+s /usr/local/clamXav/bin/freshclam)],
-		[qw(touch /usr/local/clamXav/share/clamav/freshclam.log)],
-		[qw(chmod 664 /usr/local/clamXav/share/clamav/freshclam.log)],
 		[qw(chown -R clamav:clamav /usr/local/clamXav/share/clamav)],
-		[qw(chmod -R ug+w /usr/local/clamXav/share/clamav)],
+		[qw(find /usr/local/clamXav/share/clamav -type d -exec chmod 0775 {} \;)],
+		[qw(find /usr/local/clamXav/share/clamav -type f -exec chmod 0664 {} \;)],
+		[qw(touch /usr/local/clamXav/share/clamav/freshclam.log)],
+		[qw(chmod 0664 /usr/local/clamXav/share/clamav/freshclam.log)],
 	);
 
 	1;
@@ -604,11 +633,39 @@ sub PhaseSpecifier4ClamAV4ClamXav {
 #	2. attach it.
 #	3. install from pkg file.
 #	4. detach dmg fiel.
-#	NOTE1:
+#	REQUIRED:
 #		dmg file name format is xxxx-1.2.3.dmg
 #		pkg file name format is xxxx-1.2.3.pkg
-#	NOTE2:
-#		set $Setting{phase}->{sequence} to [qw(download attach install detach)]
+#	SAMPLE CONFIGUREATION:
+#		$Setting{version}->{latest}->{command} = <mthod of your software>;
+#		$Setting{version}->{latest}->{regexp} = qr/regexp for above/;
+#		$Setting{version}->{current}->{command} = sub {
+#			my  $newestVer;
+#
+#			opendir DFH, '/Library/Receipts' or die;
+#	        while (my $fname = readdir DFH) {
+#				if ($fname =~ m/^$Setting{option}->{name}-(\d+(?:\.\d+)*)\.pkg$/o) {
+#					my  $foundVer = $1;
+#
+#					if (
+#						!defined($newestVer)
+#							or
+#						&{$Setting{version}->{compare}}($newestVer, $foundVer) < 0
+#					) {
+#						$newestVer = $foundVer;
+#					}
+#				}
+#			}
+#			closedir DFH;
+#
+#			return $newestVer;
+#		};
+#		$Setting{version}->{current}->{regexp} = qr/^(\d+(?:\.\d+)*)\s*$/;
+#		$Setting{environment}->{PATH} = '/bin:/sbin:/usr/bin:/usr/sbin';
+#		$Setting{option}->{src} = '<Base URL for your software>';
+#		$Setting{option}->{name} = '<Name of your software>';
+#		$Setting{option}->{ext} = 'dmg';
+#		$Setting{phase}->{sequence} = [qw(download attach install detach)];
 sub PhaseSpecifier4PkgOnDmg {
 	PhaseSpecifier4General();
 	delete $Phase{extract};
@@ -627,12 +684,18 @@ sub PhaseSpecifier4PkgOnDmg {
 		],
 	];
 
+	$Phase{install}->{workdir} = undef;
 	$Phase{install}->{method} = [
 		['installer', '-pkg',
 			"/Volumes/$Setting{option}->{name}-$LatestVersion/$Setting{option}->{name}-$LatestVersion.pkg",
 			'-target', '/'
 		],
 	];
+	$Phase{install}->{rollback} = [
+		['hdiutil', 'detach',
+			"/Volumes/$Setting{option}->{name}-$LatestVersion"
+		],
+	];
 
 	$Phase{detach} = {};
 	$Phase{detach}->{workdir} = undef;
@@ -651,13 +714,18 @@ sub PhaseSpecifier4PkgOnDmg {
 #	1. download gzip compressed tar file.
 #	2. extract it.
 #	3. copy files
-#	NOTE1:
-#		set $Setting{phase}->{sequence} to [qw(download extract install)]
-#	NOTE2:
-#		set $Setting{option}->{src} to clamav-update download URL.
+#	SAMPLE CONFIGURATION:
+#		$Setting{version}->{latest}->{command} = [qw(curl --silent http://clamav-update.sourceforge.jp/release/clamav-update-2/version.txt)];
+#		$Setting{version}->{latest}->{regexp} = qr/^(\d+(?:\.\d+)*)\s*$/;
+#		$Setting{version}->{current}->{command} = [qw(clamav-update.pl --version)];
+#		$Setting{version}->{current}->{regexp} = qr/^(?s-im)clamav-update\.pl v(\d+(?:\.\d+)*)\s+.*$/;
+#		$Setting{phase}->{sequence} = [qw(download extract install)]
+#		$Setting{option}->{name} = 'clamav-update'.
+#		$Setting{option}->{src} = <one of following URL.>
 #   		'http://osdn.dl.sourceforge.jp/clamav-update'     - Tokyo Japan: OSDN Japan
 #   		'http://keihanna.dl.sourceforge.jp/clamav-update' - Keihanna Japan: Kansai Science City Internet Community
 #   		'http://qgpop.dl.sourceforge.jp/clamav-update'    - Fukuoka Japan: Kyushu GigaPOP Project
+#		$Setting{option}->{limit}->{version} = '3';
 sub PhaseSpecifier4ClamavUpdate4ClamXav {
 	PhaseSpecifier4General();
 	delete $Phase{build};
@@ -680,10 +748,10 @@ sub PhaseSpecifier4ClamavUpdate4ClamXav {
 	];
 
 	$Phase{install}->{method} = [
-		[qw(install -C -o root -g admin -m 0755 clamav-update.pl
+		[qw(install -Cbp -o root -g admin -m 0755 clamav-update.pl
 			/usr/local/clamXav/bin/clamav-update.pl
 		)],
-		[qw(install -C -o root -g admin -m 0664 clamav-update-update.conf
+		[qw(install -Cbp -o root -g admin -m 0664 clamav-update-update.conf
 			/usr/local/clamXav/etc/clamav-update-update.conf
 		)],
 	];
@@ -695,21 +763,27 @@ sub PhaseSpecifier4ClamavUpdate4ClamXav {
 #	1. download gzip compressed tar file.
 #	2. extract it.
 #	3. copy files
-#	NOTE1:
-#		set $Setting{phase}->{sequence} to [qw(download extract install)]
-#	NOTE2:
-#		set $Setting{option}->{src} to clamav-update download URL.
+#	SAMPLE CONFIGURATION:
+#		$Setting{version}->{latest}->{command} = [qw(curl --silent http://clamav-update.sourceforge.jp/release/clamav-update-2/version.txt)];
+#		$Setting{version}->{latest}->{regexp} = qr/^(\d+(?:\.\d+)*)\s*$/;
+#		$Setting{version}->{current}->{command} = [qw(clamav-update.pl --version)];
+#		$Setting{version}->{current}->{regexp} = qr/^(?s-im)clamav-update\.pl v(\d+(?:\.\d+)*)\s+.*$/;
+#		$Setting{enveronment}->{PATH} = '/bin:/sbin:/usr/bin:/usr/sbin:usr/local/bin';
+#		$Setting{phase}->{sequence} = [qw(download extract install)]
+#		$Setting{option}->{name} = 'clamav-update'.
+#		$Setting{option}->{src} = <one of following URL.>
 #   		'http://osdn.dl.sourceforge.jp/clamav-update'     - Tokyo Japan: OSDN Japan
 #   		'http://keihanna.dl.sourceforge.jp/clamav-update' - Keihanna Japan: Kansai Science City Internet Community
 #   		'http://qgpop.dl.sourceforge.jp/clamav-update'    - Fukuoka Japan: Kyushu GigaPOP Project
+#		$Setting{option}->{limit}->{version} = '3';
 sub PhaseSpecifier4ClamavUpdate {
-	PhaseSpecifier4ClamAV4ClamXav();
+	PhaseSpecifier4ClamavUpdate4ClamXav();
 
 	$Phase{install}->{method} = [
-		[qw(install -C -o root -g root -m 0755 clamav-update.pl
+		[qw(install -bp -o root -g root -m 0755 clamav-update.pl
 			/usr/local/bin/clamav-update.pl
 		)],
-		[qw(install -C -o root -g root -m 0664 clamav-update-update.conf
+		[qw(install -bp -o root -g root -m 0664 clamav-update-update.conf
 			/usr/local/etc/clamav-update-update.conf
 		)],
 	];
@@ -721,13 +795,14 @@ sub PhaseSpecifier4ClamavUpdate {
 #-------------------------------------------------------------------------------
 # グローバル変数初期値
 #-------------------------------------------------------------------------------
-$Version = 'clamav-update.pl v2.0';
+$Version = 'clamav-update.pl v2.1.1';
 
 %ErrorCode = (
 	arguments		=> 1,
 	prepare			=> 2,
 	loadConfig		=> 3,
-	phaseSpecify	=> 4,
+	version			=> 4,
+	phaseSpecify	=> 5,
 	fastPhase		=> 8,
 );
 
@@ -778,6 +853,10 @@ $SyslogOpened = 0;
 		name	=> 'clamav',
 		ext		=> 'tar.gz',
 		force	=> 0,
+		limit	=> {
+			version	=> undef,
+			action	=> undef,
+		},
 	},
 	phase	=> {
 		sequence	=> [qw(download extract build install)],
@@ -794,6 +873,10 @@ $SyslogOpened = 0;
 	dst		=> undef,
 	name	=> undef,
 	ext		=> undef,
+	limit	=> {
+		version	=> undef,
+		action	=> undef,
+	},
 );
 
 
@@ -822,6 +905,37 @@ DEFAULT: /usr/local/clamXav/etc/clamav-update.conf
 
 Force install latest version.
 
+=item -s I<donwload_from> | --src I<donwload_from>
+
+Change download base URL to I<donwload_from>.
+DEGAULT: http://jaist.dl.sourceforge.net/sourceforge/clamav
+
+=item -d I<download_to> | --dst I<download_to>
+
+Change directory of destination to I<donwload_from>.
+DEFAULT: Random named directory under system temporary directory.
+
+=item -n I<package_name> | --name I<package_name>
+
+Change package name to I<package_name>.
+DEFAULT: clamav
+
+=item -e I<package_extention> | --ext I<package_extention>
+
+Change package file extention to I<package_extention>.
+DEFAULT: tar.gz
+
+=item --limit-version I<version_number>
+
+Change version that is limmited update to I<version_number>.
+DEFAULT: NONE
+
+=item --limit-action I<phase_name>
+
+Change phase name for actions when update is limited to I<phase_name>.
+DEFAULT: NONE
+
+
 =back
 
 =cut
@@ -841,12 +955,14 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 EOF
 		exit 0;
 	},
-	'c|config=s'	=> \$Option{config},
-	'f|force'		=> \$Option{force},
-	's|src=s'		=> \$Option{src},
-	'd|dst=s'		=> \$Option{dst},
-	'n|name=s'		=> \$Option{name},
-	'e|ext=s'		=> \$Option{ext},
+	'c|config=s'		=> \$Option{config},
+	'f|force'			=> \$Option{force},
+	's|src=s'			=> \$Option{src},
+	'd|dst=s'			=> \$Option{dst},
+	'n|name=s'			=> \$Option{name},
+	'e|ext=s'			=> \$Option{ext},
+	'limit-version=s'	=> \$Option{limit}->{version},
+	'limit-action=s'	=> \$Option{limit}->{action},
 ) or pod2usage($ErrorCode{arguments});
 
 
@@ -859,9 +975,19 @@ unless (CheckSecurity()) {
 	exit $ErrorCode{prepare};
 }
 
-# バージョン取得
+# 最新バージョン取得
 $LatestVersion = LatestVersion();
+unless (defined($LatestVersion)) {
+	Log 'err', 'Can\'t check latest version.';
+	exit $ErrorCode{version};
+}
+
+# カレントバージョン取得
 $CurrentVersion = CurrentVersion();
+unless (defined($CurrentVersion)) {
+	Log 'err', 'Can\'t check current version.';
+	exit $ErrorCode{version};
+}
 
 # アップデート
 if (&{$Setting{version}->{compare}}($CurrentVersion, $LatestVersion) < 0) {
@@ -883,29 +1009,55 @@ if (&{$Setting{version}->{compare}}($CurrentVersion, $LatestVersion) < 0) {
 		PhaseSpecifier4ClamAV4ClamXav();
 	}
 
-	# フェーズ実行
-	if (defined($Setting{phase}->{sequence})) {
-		if (ref($Setting{phase}->{sequence}) eq 'ARRAY') {
-			for (my $i = 0; $i < scalar(@{$Setting{phase}->{sequence}}); $i++) {
-				unless (Do($Setting{phase}->{sequence}->[$i])) {
-					exit($ErrorCode{fastPhase} + $i);
-				}
+	my	$limit = GetOptionValue(qw(limit version));
+
+	# バージョンアップ制限検査
+	if (
+		defined($limit)
+			and
+		&{$Setting{version}->{compare}}($limit, $LatestVersion) <= 0
+	) {
+		# 制限時指定フェーズ実行
+		Log 'warning', 'auto update is limited: %s <= %s',
+			$limit, $LatestVersion
+		;
+		$limit = GetOptionValue(qw(limit action));
+		if (defined($limit)) {
+			unless (Do($limit)) {
+				exit($ErrorCode{fastPhase});
 			}
 		}
-		else {
-			Log 'crit', 'unsupported phase sequence type: %s',
-				ref($Setting{phase}->{sequence}) eq ''
-					? 'ATOM'
-					: ref($Setting{phase}->{sequence})
-			;
-			exit $ErrorCode{phaseSpecify};
-		}
 	}
 	else {
-		Log 'warning', 'phase sequence not defined';
+		# フェーズ実行
+		if (defined($Setting{phase}->{sequence})) {
+			if (ref($Setting{phase}->{sequence}) eq 'ARRAY') {
+				for (
+					my $i = 0;
+					$i < scalar(@{$Setting{phase}->{sequence}});
+					$i++
+				) {
+					unless (Do($Setting{phase}->{sequence}->[$i])) {
+						exit($ErrorCode{fastPhase} + $i);
+					}
+				}
+			}
+			else {
+				Log 'crit', 'unsupported phase sequence type: %s',
+					ref($Setting{phase}->{sequence}) eq ''
+						? 'ATOM'
+						: ref($Setting{phase}->{sequence})
+				;
+				exit $ErrorCode{phaseSpecify};
+			}
+		}
+		else {
+			Log 'warning', 'phase sequence not defined';
+		}
 	}
 }
 
+
 # 終了処理
 END {
 	Log 'info', 'Finish clamav-update.pl --config %s', GetOptionValue('config');
@@ -918,6 +1070,23 @@ END {
 
 =over 4
 
+=item v2.1
+
+ * Add new feature "update limitting". 
+ * Add new optinos description to POD.
+ * Improve build-in phase specifier functions.
+   Now you can use PhaseSpecifier4ClamavUpdate on the system which use GNU
+   install command as standard install comannd.
+
+=item v2.0.1
+
+ * Add -b option to install command in built-in phase specifier.
+ * Handle the error of getting latest version, and terminate if it occurs.
+ * Allocate 4 as exit-status code for error in getting latest version.
+ * Reallocate 5 as exit-status code for error in phase specifing.
+ * workdir of the phase install is corrected to undef in PhaseSpecifier4PkgOnDmg.
+ * Add rollback of the phase install in PhaseSpecifier4PkgOnDmg.
+
 =item v2.0
 
  * Each phase sepecification is re-designed for general purpose.
